/*! \file MarsSDK.h
    \author Cooper Lawrence
    \brief Definitions for MarsSDK
*/

#pragma once

#if _WIN32
#	define EXPORT __declspec(dllexport)
#	define IMPORT __declspec(dllimport)
#else
#	define EXPORT __attribute__((visibility("default")))
#   define IMPORT __attribute__((visibility("default")))
#endif

#if defined(COMPILING_DLL)
#	define DLL_FUNCTION EXPORT
#else
#	define DLL_FUNCTION IMPORT
#endif

#include <libTof.h>

/*! \class Stream
	\brief Parent class used for image streams from the camera.
*/
class DLL_FUNCTION Stream
{
protected:
	/*! \var CLibTof* camera
		\brief Instance of camera in external Sunny Mars API.
	*/
	CLibTof* camera;

	/*! \var int width
		\brief Stream's width, in pixels.
	*/
	int width;

	/*! \var int height
		\brief Stream's height, in pixels.
	*/
	int height;

	/*! \var in size
		\brief Stream's size, in pixels.
	*/
	int size;

public:
	/*! \fn int getWidth()
		\brief Returns the Stream's width, in pixels.
		\return \link width \endlink
	*/
	int getWidth();

	/*! \fn int getHeight()
		\brief Returns the Stream's height, in pixels.
		\return \link height \endlink
	*/
	int getHeight();

	/*! \fn int getSize()
		\brief Returns the Stream's size, in pixels.
		\return \link size \endlink
	*/
	int getSize();
};

/*! \class DepthStream
	\brief Depth image stream from the camera.
*/
class DLL_FUNCTION DepthStream : public Stream
{
	friend class Mars;
private:
	/*! \fn DepthStream DepthStream(CLibTof* _camera)
		\brief Constructor to create a new instance of DepthStream.
		\param[in] _camera Instance of camera.
	*/
	DepthStream(CLibTof* _camera);

	/*! \var float* depthFrame
		\brief Depth frame buffer.
		Pixel elements can be accessed as (location * 3) + axis, where axis is 0 = x, 1 = y, 2 = z.
	*/
	float* depthFrame;

	/*! \var unsigned char* colorFrame
		\brief Depth frame color from image registration.
		Pixel elements can be accessed as (location * 3) + color, where color is 0 = r, 1 = g, 2 = b.
	*/
	unsigned char* colorFrame;

	/*! \var unsigned short* irFrame
		\brief Infrared frame buffer
		Each pixel is stored as 16-bit grayscale
	*/
	unsigned short* irFrame;

	/*! \var unsigned char* pseudoColor
		\brief Depth frame pseudo color.
		RGBA, with constant alpha = 0.
	*/
	//unsigned char* pseudoColor;

public:
	/*! \fn DepthStream ~DepthStream()
		\brief Destructor to delete memory allocation for instance of DepthStream.
	*/
	~DepthStream();

	/*! \fn int update()
		\brief Updates the stream's contents from the camera.
        	\return Error code
    */
    int update();

	/*! \fn int updatePseudoColor(int _min, int _max)
		\brief Updates the stream's pseudocolor contents.
		Uses the frame from the latest call to DepthStream::update().
		\param[in] _min Minimum distance for pseudo color calculation.
		\param[in] _max Maximum distance for pseudo color calculation.
		\return Error code
	*/
	//int updatePseudoColor(float _min, float _max);

	/*! \fn float* getFrame()
		\brief Returns the depth data.
		Uses the frame from the latest call to DepthStream::update().
		\return \link depthFrame \endlink
	*/
	float* getFrame();

	/*! \fn unsigned char* getColorFrame()
		\brief Returns the depth frame's color, calculated from the ColorStream's data through image registration.
		Uses the frame from the latest call to DepthStream::update().
		\return \link colorFrame \endlink
	*/
	unsigned char* getColorFrame();

	/*! \fn unsigned short* getIrFrame()
		\brief Returns the infrared data.
		Uses the frame from the latest call to DepthStream::update().
		\return \link irFrame \endlink
	*/
	unsigned short* getIrFrame();

	/*! \fn unsigned char* getPseudoColorFrame()
		\brief Returns the depth frame's pseudo color.
		Uses the frame from the latest call to DepthStream::updatePseudoColor().
		\return \link pseudoColor \endlink
	*/
	//unsigned char* getPseudoColorFrame();
};

/*! \class ColorStream
	\brief Color image stream from the camera.
*/
class DLL_FUNCTION ColorStream : public Stream
{
	friend class Mars;
private:
	/*! \fn ColorStream ColorStream(CLibTof* _camera)
		\brief Constructor to create a new instance of ColorStream.
		\param[in] _camera Instance of camera.
	*/
	ColorStream(CLibTof* _camera);

	/*! \var short* frame
		\brief Color frame buffer
		In YUV420 format.
	*/
	short* frame;

public:
	/*! \fn ColorStream ~ColorStream()
		\brief Destructor to delete memory allocation for instance of ColorStream.
	*/
	~ColorStream();

	/*! \fn int update()
		\brief Updates the stream's contents from the camera.
		\return Error code
        */
        int update();

	/*! \fn short* getFrame()
		\brief Returns the color data.
		Uses the frame from the latest call to ColorStream::update().
		\return \link frame \endlink
	*/
	short* getFrame();
};

/*! \class Mars
	\brief Sunny Mars device.
*/
class DLL_FUNCTION Mars
{
private:
	/*! \var CLibTof* camera
		\brief Instance of camera in external Sunny Mars API
	*/
	CLibTof* camera;

	/*! \var DeviceInfo_t* deviceInfo
		\brief Structure containing all the device-specific information, retrieved from the Sunny Mars API.
	*/
	DeviceInfo_t* deviceInfo;

	/*! \var DepthStream* depthStream
		\brief Instance of DepthStream within MarsSDK
	*/
	DepthStream* depthStream;

	/*! \var ColorStream* colorStream
		\brief Instance of ColorStream within MarsSDK
	*/
	ColorStream* colorStream;

public:
	/*! \fn Mars Mars()
		\brief Constructor to create a new instance of Mars.
	*/
	Mars();

	/*! \fn Mars ~Mars()
		\brief Destructor to delete memory allocation for instance of Mars.
	*/
	~Mars();

	/*! \fn int connect()
		\brief Connects the connected camera.
		\return Error code
	*/
	int connect();

	/*! \fn int disconnect()
		\brief Disconnects the connected camera.
		\return Error code
	*/
	int disconnect();

	/*! \fn DepthStream* getDepthStream()
		\brief Creates a new instance of DepthStream, if one doesn't already exist.
		\return \link depthStream \endlink
	*/
	DepthStream* getDepthStream();

	/*! \fn ColorStream* getColorStream()
		\brief Creates a new instance of ColorStream, if one doesn't already exist.
		\return \link colorStream \endlink
	*/
	ColorStream* getColorStream();

	/*! \fn int closeDepthStream()
		\brief Deletes the instance of DepthStream, if one already exists.
		\return Error code
	*/
	int closeDepthStream();

	/*! \fn int closeColorStream()
		\brief Deletes the instance of ColorStream, if one already exists.
		\return Error code
	*/
	int closeColorStream();

	/*! \fn unsigned int getDeviceVersion()
		\brief Returns the device's version.
		\return Device version
	*/
	unsigned int getDeviceVersion();

	/*! \fn unsigned int getDeviceType()
		\brief Returns the device type.
		\return Device type
	*/
	unsigned int getDeviceType();

	/*! \fn char* getDeviceId()
		\brief Returns the device ID.
		\return Device ID
	*/
	char* getDeviceId();

	/*! \fn char* getDriverVersion()
		\brief Returns the driver version.
		\return Driver version
	*/
	char* getDriverVersion();

	/*! \fn int getTemp(float* _cpu, float* _sensor)
		\brief Gets the camera's temperature.
		\param[out] _cpu Camera's CPU temperature.
		\param[out] _sensor Camera's sensor temperature.
		\return Error code
	*/
	int getTemp(float *_cpu, float *_sensor);

	/*! \fn int setCameraMode(int _mode)
		\brief Sets the working mode for the camera's time of flight (TOF) sensor.
		0 = mono frequency, 1 = multi frequency, 2 = IR data
		\param[in] _mode Desired TOF mode.
		\return Error code
	*/
	int setCameraMode(int _mode);

	/*! \fn int setFps(int _fps)
		\brief Sets the camera's framerate.
		Options available are 5, 15, 30, and 45 fps.
		\param[in] _fps Desired framerate.
		\return Error code
	*/
	int setFps(int _fps);

	/*! \fn int setExposure(int _exposure)
		\brief Sets the camera's exposure.
		Options available are 0 (automatic), 250, 380, 650, 850, and 1200.
		\param[in] _exposure Desired exposure.
		\return Error code
	*/
	int setExposure(int _exposure);

	/*! \fn int setIrExposure(int _exposure)
		\brief Sets the infrared camera's exposure.
		Options available are 0 (off) through 1200.
		\param[in] _exposure Desired exposure.
		\return Error code
	*/
	//int setIrExposure(int _exposure);

	/*! \fn int setImageRegistration(bool _status)
		\brief Sets the status of the camera's image registration between the depth stream and color stream.
		\param[in] _status Enable/ disable image registration.
		\return Error code
	*/
	int setImageRegistration(bool _status);
};
